from tkinter.font import *


class KiFont(object):
    def __init__(self, Root=None, Family: str = "Cascadia Mono", Size: int = 10, Weight: str = "light"):
        self.Widget = Font(root=Root, family=Family, size=Size, weight=Weight)

    def Families(self, Root=None):
        return families(root=Root)

    def CGet(self, Option):
        self.Widget.cget(option=Option)

    def Config(self, Root=None, Family: str = "Cascadia Mono", Size: int = 10, Weight: str = "light"):
        self.Widget.config(root=Root, family=Family, size=Size, weight=Weight)

    def GetFont(self):
        return self.Widget


from tkinter import *
import tkinter

def EmptyFunc():
    pass

EmptyString = ""


# icons
Message_Icons_ERROR = "error"
Message_Icons_INFO = "info"
Message_Icons_QUESTION = "question"
Message_Icons_WARNING = "warning"

# types
Message_Type_ABORTRETRYIGNORE = "abortretryignore"
Message_Type_OK = "ok"
Message_Type_OKCANCEL = "okcancel"
Message_Type_RETRYCANCEL = "retrycancel"
Message_Type_YESNO = "yesno"
Message_Type_YESNOCANCEL = "yesnocancel"

# replies
Message_ABORT = "abort"
Message_RETRY = "retry"
Message_IGNORE = "ignore"
Message_OK = "ok"
Message_CANCEL = "cancel"
Message_YES = "yes"
Message_NO = "no"

# Edit
Edit_Modified = "edit_modified"
Edit_Undo = "edit_undo"
Edit_Redo = "edit_redo"
Edit_Reset = "edit_reset"
Edit_Separator = "edit_separator"
Insert = INSERT
End = END

# Warp
Wrap_Word = WORD
Wrap_Char = CHAR
Wrap_None = NONE

# TtkStyle
Style_SunValley = "Style-SunValley"
Style_Azure = "Style-Azure"
Theme_Light = "light"
Theme_Dark = "dark"
Button_Accent = "Accent.TButton"

# SideBarSide
SideBarSide_Left = LEFT
SideBarSide_Right = RIGHT

# Attributes
Window_Maximize = "-fullscreen"
Window_Topmost = "-topmost"
Window_Delete = "WM_DELETE_WINDOW"
Window_Save_YouSelf = "WM_SAVE_YOURSELF"

# Orient
Orient_Horizontal = HORIZONTAL
Orient_Vertical = VERTICAL

# Relief "raised", "sunken", "flat", "ridge", "solid", "groove"
Relief_Raised = RAISED
Relief_Sunken = SUNKEN
Relief_Flat = FLAT
Relief_Ridge = RIDGE
Relief_Solid = SOLID
Relief_Groove = GROOVE

# Jusitfy "left", "center", "right"
Jusitfy_Left = LEFT
Jusitfy_Center = CENTER
Jusitfy_Right = RIGHT

# Anchor "nw", "n", "ne", "w", "center", "e", "sw", "s", "se"
Anchor_Nw = NW
Anchor_N = N
Anchor_Ne = NE
Anchor_W = W
Anchor_Center = CENTER
Anchor_E = E
Anchor_Sw = SW
Anchor_S = S
Anchor_Se = SE

# Fill "none", "both", "x", "y"
Fill_None = NONE
Fill_BOTH = BOTH
Fill_X = X
Fill_Y = Y

# Expand "yes", "no"
Expand_YES = YES
Expand_NO = NO

# Side "left", "right", "top", "bottom"
Side_Left = LEFT
Side_Right = RIGHT
Side_Top = TOP
Side_Bottom = BOTTOM

# Bind

class KiVar(object):
    def SetVar(self, Value):
        self.Widget.set(Value)

    def GetVar(self):
        return self.Widget.get()

    @property
    def Var(self):
        return self.Widget

    def Get(self):
        return self.Widget


class KiStringVar(KiVar):
    def __init__(self):
        self.Widget = StringVar()


class KiIntVar(KiVar):
    def __init__(self):
        self.Widget = IntVar()


class KiBooleanVar(KiVar):
    def __init__(self):
        self.Widget = BooleanVar()


from Tki.KiCore import *


class KiData(object):
    def __init__(self, InitialData: dict = {}):
        """
        :param InitialData:
        KiData是用于储存类里的一些数据，为字典数字格式。
        """
        self.MasterData = InitialData

    def SetData(self, DataName: str, DataValue: str):
        self.MasterData[DataName] = DataValue

    def GetData(self, DataName: str):
        return self.MasterData[DataName]

    def GetMasterData(self):
        return self.MasterData[self.MasterData]


class KiMisc(object):
    def __init__(self):
        self.Widget = Misc()


class KiCilpBoard(object):
    def __init__(self):
        self.Widget = None

    def CilpBoardAppend(self, String):
        """
        :param String:
        添加剪切板文字
        """
        self.Widget.clipboard_append(string=String)

    def CilpBoardClear(self):
        """
        清空剪切板文字
        """
        self.Widget.clipboard_clear()

    def CilpBoardGet(self):
        return self.Widget.clipboard_get()


class KiBasic(object):
    def __init__(self, Master: any = None, WigetName: str = ""):
        """
        KiBasic -基本组件，所有可视化组件的父类
                :param Master: 父组件
        """
        try:
            self.Widget = BaseWidget(Master, WigetName)
        except tkinter.TclError:
            pass

    @property
    def Tk(self):
        return self.Widget.tk

    def Tcl(self, Command, Argv):
        self.Widget.tk.call(Command, Argv)

    def GetConfig(self):
        return self.Widget.config

    def SetSource(self, Source):
        self.Tcl("source", Source)

    def SetTtkStyle(self, Style, Theme):
        if Style == "Style-SunValley":
            self.SetSource(f"{getcwd()}\Tki\SunValley\sun-valley.tcl")
        elif Style == "Style-Azure":
            self.SetSource(f"{getcwd()}\Tki\Azure\azure.tcl")
        self.Tcl("set_theme", Theme)

    def GridAnchor(self, Anchor=None):
        self.Widget.grid_anchor(anchor=Anchor)

    def GridBBox(self, Column=None, Row=None, Column2=None, Row2=None):
        """
        :param Column:
        :param Row:
        :param Column2:
        :param Row2:
        返回边界的整数坐标元组此小部件的框由几何管理器网格控制。如果给定 COLUMN, ROW，则边界框适用于具有指定行和列 0 的单元格细胞。 如果给定边界框 COL2 和 ROW2从那个单元格开始。
        返回的整数指定左上角的偏移量
        主小部件中的角以及宽度和高度。
        """
        self.Widget.grid_bbox(column=Column, row=Row, col2=Column2, row2=Row2)

    def After(self, Time, Headle):
        """
        :param Time:
        :param Headle:
        After 以毫秒为单位，每隔一段事件执行一次事件，会返回一个Id，使用AfterCanel进行取消。
        Demo: Id = After(50, Func)  # 开启循环并获取返回ID
              AfterCanel(Id)  # 取消循环
        """
        try:
            self.AfterID = self.Widget.after(ms=Time, func=Headle)
        except TypeError as Error:
            print(Error)
        return self.AfterID

    def AfterCanel(self, AfterID: str):
        """
        :param AfterID: After的返回值。
        AfterCanel 取消After命令，需输入参数AfterID
        """
        try:
            self.Widget.after_cancel(id=AfterID)
        except TypeError as Error:
            print(Error + "\n请确认参数是否是字串符，并且是After的返回数值")

    def AfterIdle(self, Headle):
        """
        :param Headle:
        AfterIdle 如果 Tcl 主循环没有事件，会自动调用Headle函数。
        """
        try:
            self.Widget.after_idle(func=Headle)
        except TypeError as Error:
            print(Error + "\n请确认是否输入参数且为函数")

    def Destory(self):
        """
        Destory 销毁当前组件
        """
        self.Widget.destroy()

    def Anchor(self, Anchor=None):
        self.GridAnchor(Anchor)

    def Configure(self, Cnf):
        self.Widget.configure(cnf=Cnf)

    def CommandDelete(self, Name: str):
        self.Widget.deletecommand(Name)

    def UpDate(self):
        self.Widget.update()

    def CGet(self, Key: str):
        return self.Widget.cget(Key)

    def Bell(self, Display: bool = True):
        """
        :param Display:
        播放系统应用，Display决定是否播放声音
        """
        self.Widget.bell(displayof=Display)

    def Bind(self, BindName=None, Headle=None, Add: bool = None):
        """
        :param BindName:
        :param Headle:
        :param Add:
        BindName为事件名称，Headle为事件函数，Add抉择是否将事件函数被附加到其他绑定函数或是否它将取代以前的功能。将返回一个标识符用来取消事件.
        """
        return self.Widget.bind(sequence=BindName, func=Headle, add=Add)

    def BindAll(self, BindName=None, Headle=None, Add: bool = None):
        """
        :param BindName:
        :param Headle:
        :param Add:
        BindName为事件名称，Headle为事件函数，Add抉择是否将事件函数被附加到其他绑定函数或是否它将取代以前的功能。将返回一个标识符用来取消事件,不同的是将绑定到所有组件上面
        """
        return self.Widget.bind(sequence=BindName, func=Headle, add=Add)

    def BindTags(self):
        return self.Widget.bindtags()

    def UnBind(self, BindName=None, HeadleFuncId=None):
        self.Widget.unbind(sequence=BindName, funcid=HeadleFuncId)

    def UnBindAll(self, BindName):
        self.Widget.unbind_all(sequence=BindName)

    def WaitWindowDestory(self, Window):
        self.Widget.wait_window()

    def BBox(self, Column=None, Row=None, Column2=None, Row2=None):
        self.GridBBox(Column, Row, Column2, Row2)

    def Quit(self):
        self.Widget.quit()

    def MainLoop(self):
        self.Widget.mainloop()


class KiGet(object):
    def __init__(self):
        self.Widget = None

    def GetClass(self):
        return self.Widget.winfo_class()

    def GetChild(self):
        return self.Widget.winfo_children()

    def GetContaining(self, RootX, RootY, DisplayOf=None):
        return self.Widget.winfo_containing(rootX=RootX, rootY=RootY, displayof=DisplayOf)

    def GetExists(self):
        return self.Widget.winfo_exists()

    def GetGeometry(self):
        return self.Widget.winfo_geometry()

    def GetHeight(self):
        return self.Widget.winfo_height()

    def GetScreenWidth(self):
        return self.Widget.winfo_screenwidth()

    def GetScreenHeight(self):
        return self.Widget.winfo_screenheight()

    def GetWidth(self):
        return self.Widget.winfo_width()

    def GetID(self):
        return self.Widget.winfo_id()

    def GetInterps(self, DisplayOf=0):
        return self.Widget.winfo_interps(displayof=DisplayOf)

    def GetManager(self):
        return self.Widget.winfo_manager()

    def GetName(self):
        return self.Widget.winfo_name()

    def GetParent(self):
        return self.Widget.winfo_parent()

    def GetPathName(self, ID, DisplayOf=0):
        return self.Widget.winfo_pathname(id=ID, displayof=DisplayOf)

    def GetPointerX(self):
        return self.Widget.winfo_pointerx()

    def GetPointerY(self):
        return self.Widget.winfo_pointery()

    def GetPointerXY(self):
        return self.Widget.winfo_pointerxy()

    def GetRootX(self):
        return self.Widget.winfo_rootx()

    def GetRootY(self):
        return self.Widget.winfo_rooty()

    def GetScreen(self):
        return self.Widget.winfo_screen()

    def GetToplevel(self):
        return self.Widget.winfo_toplevel()

    def GetX(self):
        return self.Widget.winfo_x()

    def GetY(self):
        return self.Widget.winfo_y()


class KiPack(object):
    def __init__(self):
        self.Widget = Pack()

    def PackWidget(self, Padx=0, Pady=0, iPadx=0, iPady=0,
                   Side=Side_Top, Anchor: str = Anchor_Nw, Fill: str = Fill_None, Expand=Expand_NO):
        self.Widget.pack(padx=Padx, pady=Pady, ipadx=iPadx, ipady=iPady,
                         anchor=Anchor, fill=Fill, expand=Expand, side=Side)

    def PackInfo(self):
        return self.Widget.pack_info()

    def PackForget(self):
        self.Widget.pack_forget()

    def PackConfigure(self, Padx=0, Pady=0, iPadx=0, iPady=0,
                      Side=Side_Top, Anchor: str = Anchor_Nw, Fill: str = Fill_None, Expand=Expand_NO):
        self.Widget.pack_configure(padx=Padx, pady=Pady, ipadx=iPadx, ipady=iPady,
                                   anchor=Anchor, fill=Fill, expand=Expand, side=Side)


class KiPlace(object):
    def __init__(self):
        self.Widget = Place()

    def PlaceWidget(self, X: int, Y: int, Width: int, Height: int, Anchor: str = Anchor_Nw):
        self.Widget.place(x=str(X), y=str(Y), width=str(Width), height=str(Height), anchor=Anchor)

    def PlaceInfo(self):
        return self.Widget.place_info()

    def PlaceForget(self):
        self.Widget.place_forget()

    def PlaceConfigure(self, X: int, Y: int, Width: int, Height: int, Anchor: str = Anchor_Nw):
        self.Widget.place_configure(x=str(X), y=str(Y), width=str(Width), height=str(Height), anchor=Anchor)


class KiGrid(object):
    def __init__(self):
        self.Widget = Grid()

    def GridWidget(self, Row: int, Column: int, Sticky: str = "", Padx=0, Pady=0, iPadx=0, iPady=0):
        self.Widget.grid(row=Row, column=Column, padx=Padx, pady=Pady, ipadx=iPadx, ipady=iPady, sticky=Sticky)

    def GridInfo(self):
        return self.Widget.grid_info()

    def GridForget(self):
        self.Widget.grid_forget()


class KiTheme(object):
    def __init__(self):
        pass

class KiObject(KiCilpBoard, KiBasic, KiGet, KiData):
    def __init__(self):
        super().__init__()
        self.Widget = None

    def SetWidget(self, Widget):
        self.Widget = Widget

    def GetWidget(self):
        return self.Widget



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Toolkit-InterFace",
    version="0.3 beta",
    author="XiangQinxi",
    author_email="XiangQinxi@outlook.com",
    description="Tkinter Box",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/XiangQinxi/Tki",
    project_urls={
        "Bug Tracker": "https://github.com/XiangQinxi",
    },
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Tcl",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 4 - Beta",
        "License :: Freeware",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: Microsoft",
        "Operating System :: Unix",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)


from tkinter import *
from tkinter import ttk
from tkinter.messagebox import *
from tkinter.simpledialog import *
from colorama import Fore, Style

try:
    from tkinterie.tkinterIE import WebView
except ImportError:
    print(Fore.RED+"Please Download tkinterie, install commnad: pip install tkinterie "+Style.RESET_ALL)
from Tki import *


class KiWidget(KiObject, KiPack, KiPlace):
    def __init__(self):
        super().__init__()
        try:
            self.Widget = Widget()
        except TypeError:
            pass

    def Init(self):
        self.SetRelief(Relief_Ridge)

    def SetMaster(self, Master):
        self.Widget.configure(master=Master)

    def SetState(self, State: str):
        self.Widget.configure(state=State)

    def SetRelief(self, Relief):
        self.Widget.configure(relief=Relief)

    def SetBackGround(self, Background):
        self.Widget.configure(background=Background)

    def SetBorder(self, Border):
        self.Widget.configure(Border)


class KiWindow(KiWidget):
    def __init__(self):
        super().__init__()
        self.Widget = Tk()
        self.SetTitle("Tki")
        self.Resize(500, 500)
        self.Init()

    def Attributes(self, Option: str, Value):
        self.Widget.attributes(Option, Value)

    def Maxmize(self, If: bool = True):
        self.Attributes(Window_Maximize, If)

    def Topmost(self, If: bool = True):
        self.Attributes(Window_Topmost, If)

    def Protocol(self, Name=None, Func=None):
        self.Widget.protocol(name=Name, func=Func)

    def SetMaxSize(self, Width, Height):
        self.Widget.maxsize(Width, Height)

    def SetMinSize(self, Width, Height):
        self.Widget.minsize(Width, Height)

    def SetTitle(self, Title: str):
        self.Widget.title(string=Title)

    def SetIcon(self, Icon):
        self.Widget.iconbitmap(bitmap=Icon)

    def Quit(self):
        self.Widget.quit()

    def Resize(self, Width: int, Height: int):
        self.Widget.geometry(f"{Width}x{Height}")

    def Center(self):
        X = round(int(self.GetScreenWidth()) / 2) + round(int(self.GetWidth()) / 2)
        Y = round(int(self.GetScreenHeight()) / 2) + round(int(self.GetHeight()) / 2)
        self.Move(X, Y)

    def Move(self, X: int = None, Y: int = None):
        self.Widget.geometry(f"+{X}+{Y}")

    def UpDate(self):
        self.Widget.update()

    def Run(self):
        self.MainLoop()


class KiToplevel(KiWidget):
    def __init__(self):
        super().__init__()
        self.Widget = Toplevel()
        self.SetTitle("Toplevel")
        self.Resize(500, 500)
        self.Init()

    def Attributes(self, Option: str, Value):
        self.Widget.attributes(Option, Value)

    def Maxmize(self, If: bool = True):
        self.Attributes(Window_Maximize, If)

    def Topmost(self, If: bool = True):
        self.Attributes(Window_Topmost, If)

    def Protocol(self, Name=None, Func=None):
        self.Widget.protocol(name=Name, func=Func)

    def SetMaxSize(self, Width, Height):
        self.Widget.maxsize(Width, Height)

    def SetMinSize(self, Width, Height):
        self.Widget.minsize(Width, Height)

    def SetTitle(self, Title: str):
        self.Widget.title(string=Title)

    def SetIcon(self, Icon):
        self.Widget.iconbitmap(bitmap=Icon)

    def Quit(self):
        self.Widget.quit()

    def Resize(self, Width: int, Height: int):
        self.Widget.geometry(f"{Width}x{Height}")

    def Center(self):
        X = round(int(self.GetScreenWidth()) / 2) + round(int(self.GetWidth()) / 2)
        Y = round(int(self.GetScreenHeight()) / 2) + round(int(self.GetHeight()) / 2)
        self.Move(X, Y)

    def Move(self, X: int = None, Y: int = None):
        self.Widget.geometry(f"+{X}+{Y}")

    def UpDate(self):
        self.Widget.update()

    def Run(self):
        self.MainLoop()


class KiScrollBar(KiWidget):
    def __init__(self, Master: KiWidget = None,
                 Background: str = "white", ActiveBackground: str = "black", Relief: str = Relief_Flat,
                 Orient: str = Orient_Vertical,
                 Width=0):
        super().__init__()
        self.Widget = Scrollbar(master=Master, background=Background, activebackground=ActiveBackground, relief=Relief,
                                width=Width, orient=Orient)

    def Configure(self, Master: KiWidget = None,
                  Background: str = "white", ActiveForeground: str = "black", Relief: str = Relief_Flat,
                  Width=0):
        self.Widget.configure(master=Master, background=Background, activebackground=ActiveBackground, relief=Relief,
                              width=Width)

    def SetValue(self, First, Last):
        self.Widget.set(first=First, last=Last)

    def GetValue(self):
        return self.Widget.get()


class KiTextView(KiWidget):
    def __init__(self, Master: KiWidget = None,
                 SelectBackground: str = "blue", SelectForeground: str = "white",
                 Background: str = "white", Foreground: str = "black", XScroll: KiScrollBar = None, Relief: str = Relief_Flat,
                 YScroll: KiScrollBar = None, Wrap: str = Wrap_Char, Font=None,
                 Width=0):
        super().__init__()
        self.Widget = Text(master=Master, selectbackground=SelectBackground, selectforeground=SelectForeground, relief=Relief,
                           background=Background, foreground=Foreground, wrap=Wrap, font=Font,
                           xscrollcommand=XScroll, yscrollcommand=YScroll, width=Width)

    def Configure(self, Master: KiWidget = None,
                  SelectBackground: str = "white", SelectForeground: str = "black",
                  Background: str = "white", Foreground: str = "black", XScroll: KiScrollBar = None, Relief: str = Relief_Flat,
                  YScroll: KiScrollBar = None, Wrap: str = Wrap_Char, Font=None,
                  Width=0):
        self.Widget.configure(master=Master, selectbackground=SelectBackground, selectforeground=SelectForeground, relief=Relief,
                              background=Background, foreground=Foreground, wrap=Wrap, font=Font,
                              xscrollcommand=XScollBar, yscrollcommand=YScroll, width=Width)

    def WidgetCreate(self, Index, Widget: KiWidget):
        self.Widget.window_create(index=Index, window=Widget)

    def WidgetConfigure(self, Index):
        self.Widget.window_configure(index=Index)

    def WidgetCGet(self, Index):
        return self.Widget.window_cget(index=Index)

    def TagConfig(self, TagName: str, Start: str, End: str = None, Background="white", Foreground="black", BorderWidth: int = 0, Justify: str = Jusitfy_Left, Font=None):
        self.Widget.tag_config(tagName=TagName, index1=Start, index2=End, background=Background, foreground=Foreground, borderwidth=BorderWidth, justify=Justify, font=Font)

    def TagAdd(self, TagName: str, Start: str, End: str = None, Background="white", Foreground="black", BorderWidth: int = 0, Justify: str = Jusitfy_Left, Font=None):
        self.Widget.tag_add(tagName=TagName, index1=Start, index2=End, background=Background, foreground=Foreground, borderwidth=BorderWidth, justify=Justify, font=Font)

    def TagRemove(self, TagName: str, Start: str, End: str = None):
        self.Widget.tag_remove(tagName=TagName, index1=Start, index2=End)

    def TagBind(self, TagName: str, Sequence, Func):
        return self.Widget.tag_bind(tagName=TagName, sequence=Sequence, func=Func)

    def TagRanges(self, TagName: str):
        self.Widget.tag_ranges(tagName=TagName)

    def TagUnBind(self, TagName: str, Sequence, FuncId=None):
        self.Widget.tag_unbind(tagName=TagName, sequence=Sequence, funcid=FuncId)

    def TagDelete(self, TagName: str):
        self.Widget.tag_delete(TagName)

    def TagCGet(self, TagName: str, Option):
        self.Widget.tag_cget(tagName=TagName, option=Option)

    def Edit(self, Flag):
        self.Widget.edit(Flag)

    def EditUndo(self):
        self.Widget.edit_undo()

    def EditRedo(self):
        self.Widget.edit_redo()

    def EditModified(self):
        self.Widget.edit_modified()

    def EditSeparator(self):
        self.Widget.edit_separator()

    def EditReset(self):
        self.Widget.edit_reset()

    def Delete(self, Start: str, End: str = None):
        self.Widget.delete(index1=Start, index2=End)

    def Insert(self, Index: str, Chars: str = None):
        self.Widget.insert(index=Index, chars=Chars)

    def See(self, Index: str):
        self.Widget.see(Index)

    def Search(self, String: str, Index: str, StopIndex=End):
        self.Widget.search(pattern=String, index=Index, stopindex=StopIndex)

    def Clear(self):
        self.Widget.delete()

    def Get(self, Start: str, End: str = None):
        self.Widget.get(index1=Start, index2=End)


class KiButton(KiWidget):
    def __init__(self, Master: KiWidget = None, Text: str = EmptyString, Command=EmptyFunc,
                 ActiveBackground: str = "white", ActiveForeground: str = "black", Font=None, Relief: str = Relief_Flat,
                 Background: str = "white", Foreground: str = "black", Justify: str = Jusitfy_Center,
                 Width=0):
        super().__init__()
        self.Widget = Button(master=Master, text=Text, command=Command,
                             activebackground=ActiveBackground, activeforeground=ActiveForeground, relief=Relief,
                             background=Background, foreground=Foreground, justify=Justify, font=Font,
                             width=Width)
        self.Init()

    def SetText(self, Text):
        self.Configure(Text=Text)

    def SetCommand(self, Func):
        self.Configure(Command=Func)

    def Configure(self, Master: KiWidget = None, Text: str = EmptyString, Command=EmptyFunc,
                  ActiveBackground: str = "white", ActiveForeground: str = "black",
                  Background: str = "white", Foreground: str = "black", Justify: str = Jusitfy_Center,
                  Width=0):
        self.Widget.configure(master=Master, text=Text, command=Command,
                              activebackground=ActiveBackground, activeforeground=ActiveForeground,
                              background=Background, foreground=Foreground, justify=Justify, width=Width)

    def Flash(self):
        self.Widget.flash()


class KiEntry(KiWidget):
    def __init__(self, Master: KiWidget = None, Text: str = EmptyString,
                 Background: str = "white", Foreground: str = "black", Font=None,
                 Justify: str = Jusitfy_Left, Width=0):
        super().__init__()
        self.Text = KiStringVar()
        self.Widget = Entry(master=Master, textvariable=self.Text.Get(), background=Background, foreground=Foreground, font=Font,
                            justify=Justify, width=Width)
        self.SetText(Text)

    def SetText(self, Text):
        self.Text.SetVar(Text)

    def GetText(self):
        return self.Text.GetVar()


class KiFrame(KiWidget):
    def __init__(self, Master: KiWidget = None, Background: str = "white",
                 Padx=0, Pady=0, iPadx=0, iPady=0, Relief: str = Relief_Sunken,
                 Width=0):
        super().__init__()
        self.Widget = Frame(master=Master, background=Background, width=Width,
                            padx=Padx, pady=Pady)

    def Configure(self, Master: KiWidget = None, Background: str = "white",
                  Padx=0, Pady=0, iPadx=0, iPady=0, Relief: str = Relief_Sunken,
                  Width=0):
        self.Widget.configure(master=Master,
                              background=Background, relief=Relief,
                              width=Width, padx=Padx, pady=Pady, ipadx=iPadx, ipady=iPady)


class KiMessage(KiWidget):
    def __init__(self, Master: KiWidget = None, Text: str = EmptyString,
                 ActiveBackground: str = "white", ActiveForeground: str = "black",
                 Background: str = "white", Foreground: str = "black", Justify: str = Jusitfy_Center, Font=None,
                 Width=0):
        super().__init__()
        self.Widget = Message(master=Master, text=Text, background=Background, foreground=Foreground, justify=Justify, font=Font,
                              width=Width)

    def Configure(self, Master: KiWidget = None, Text: str = EmptyString, Command=EmptyFunc,
                  ActiveBackground: str = "white", ActiveForeground: str = "black",
                  Background: str = "white", Foreground: str = "black", Justify: str = Jusitfy_Center,
                  Width=0):
        self.Widget.configure(master=Master, text=Text, background=Background, foreground=Foreground,
                              justify=Justify, width=Width)

    def SetText(self, Text):
        self.Configure(Text=Text)


class KiMDI(KiFrame):
    def __init__(self, Master: KiWidget = None, Background: str = "#808080",
                 Padx=0, Pady=0, iPadx=0, iPady=0, Relief: str = Relief_Sunken,
                 Width=0):
        super().__init__()
        self.Widget = Frame(master=Master, background=Background, width=Width,
                            padx=Padx, pady=Pady)


class KiTEntry(KiWidget):
    def __init__(self, Master: KiWidget = None, Text: str = EmptyString, Font=None,
                 Justify: str = Jusitfy_Left, Width=0):
        super().__init__()
        self.Text = KiStringVar()
        self.Widget = ttk.Entry(master=Master, textvariable=self.Text.Get(), justify=Justify, width=Width, font=Font)
        self.SetText(Text)

    def SetText(self, Text):
        self.Text.SetVar(Text)

    def GetText(self):
        return self.Text.GetVar()


class KiTButton(KiWidget):
    def __init__(self, Master: KiWidget = None, Text: str = EmptyString, Command=EmptyFunc, Style: str = None,
                 Width=0):
        super().__init__()
        self.Widget = ttk.Button(master=Master, text=Text, command=Command,
                                 width=Width, style=Style)

    def Configure(self, Master: KiWidget = None, Text: str = EmptyString, Command=EmptyFunc, Style: str = None,
                  Justify: str = Jusitfy_Center,
                  Width=0):
        self.Widget.configure(master=Master, text=Text, command=Command,
                              width=Width, style=Style)

    def SetAccent(self):
        self.Configure(Style="Accent.TButton")

    def SetText(self, Text):
        self.Configure(Text=Text)

    def SetCommand(self, Func):
        self.Configure(Command=Func)

    def Flash(self):
        self.Widget.flash()


class KiTFrame(KiWidget):
    def __init__(self, Master: KiWidget = None, Relief: str = Relief_Sunken, Width=0):
        super().__init__()
        self.Widget = ttk.Frame(master=Master, relief=Relief, width=Width)

    def Configure(self, Relief: str = Relief_Sunken, Width=0):
        self.Widget.configure(relief=Relief, width=Width)


class KiSideBar(KiFrame):
    def __init__(self, Master: KiWidget = None, ActiveBackground: str = "white", ActiveForeground: str = "black",
                 Background: str = "white", Foreground: str = "black", Justify: str = Jusitfy_Center,
                 Padx=0, Pady=0, iPadx=0, iPady=0,
                 Width=0):
        super().__init__()
        self.Widget = KiFrame(Master=Master.GetWidget(), ActiveBackground=ActiveBackground,
                              ActiveForeground=ActiveForeground,
                              Background=Background, Foreground=Foreground, Justify=Justify,
                              Width=Width, Padx=Padx, Pady=Pady, iPadx=iPadx, iPady=iPady)

    def Put(self, Side=SideBarSide_Left,
            Padx: int = 0, Pady: int = 0, iPadx: int = 0, iPady: int = 0, ):
        self.Widget.PackWidget(Side=Side, Fill=Fill_Y, Padx=Padx, Pady=Pady, iPadx=iPadx, iPady=iPady)


class KiWebViewIE(KiWidget):
    def __init__(self, Master: KiWidget, Width, Height, Url):
        super().__init__()
        self.Widget = WebView(Master.GetWidget(), Width, Height, Url)


class KiMessgaeBox(KiWindow):
    def __init__(self):
        super().__init__()
        self.Widget = Message()

    def ShowInfo(self, Title: str = "Info", Message: str = "This is a info dialog", Icon: str = Message_Icons_INFO, Parent=None):
        return showinfo(title=Title, message=Message, icon=Icon, parent=Parent)

    def ShowError(self, Title: str = "Error", Message: str = "This is a error dialog", Icon: str = Message_Icons_ERROR, Parent=None):
        return showerror(title=Title, message=Message, icon=Icon, parent=Parent)

    def ShowWarning(self, Title: str = "Warning", Message: str = "This is a warning dialog", Icon: str = Message_Icons_WARNING, Parent=None):
        return showwarning(title=Title, message=Message, icon=Icon, parent=Parent)

    def AskYesNo(self, Title: str = "Ask", Message: str = "This is a yesno dialog", Icon: str = Message_Icons_QUESTION, Parent=None):
        return askyesno(title=Title, message=Message, icon=Icon, parent=Parent)

    def AskOkCanel(self, Title: str = "Warning", Message: str = "This is a okcanel dialog", Icon: str = Message_Icons_QUESTION, Parent=None):
        return askokcancel(title=Title, message=Message, icon=Icon, parent=Parent)

    def AskQuestion(self, Title: str = "Warning", Message: str = "This is a question dialog", Icon: str = Message_Icons_QUESTION, Parent=None):
        return askquestion(title=Title, message=Message, icon=Icon, parent=Parent)

    def AskRetryCanel(self, Title: str = "Warning", Message: str = "This is a retrycanel dialog", Icon: str = Message_Icons_QUESTION, Parent=None):
        return askretrycancel(title=Title, message=Message, icon=Icon, parent=Parent)

    def AskYesNoCanel(self, Title: str = "Warning", Message: str = "This is a yesnocanel dialog", Icon: str = Message_Icons_QUESTION, Parent=None):
        return askyesnocancel(title=Title, message=Message, icon=Icon, parent=Parent)


class KiMessgaeSimple(KiWindow):
    def AskFloat(self, Title: str = "ask float", Prompt: str = "plase entry float"):
        return askfloat(title=Title, prompt=Prompt)

    def AskInteger(self, Title: str = "ask integer", Prompt: str = "plase entry integer"):
        return askinteger()

    def AskString(self, Title: str = "ask string", Prompt: str = "plase entry string"):
        return askstring()